# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pymaid/rpc/pb/pymaid.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1apymaid/rpc/pb/pymaid.proto\x12\rpymaid.rpc.pb\"\x9b\x03\n\x07\x43ontext\x12\x17\n\x0ftransmission_id\x18\x01 \x01(\r\x12\x36\n\x0bpacket_type\x18\x02 \x01(\x0e\x32!.pymaid.rpc.pb.Context.PacketType\x12\x37\n\x0cpacket_flags\x18\x03 \x01(\x0e\x32!.pymaid.rpc.pb.Context.PacketFlag\x12\x31\n\x08priority\x18\x04 \x01(\x0e\x32\x1f.pymaid.rpc.pb.Context.Priority\x12\x16\n\x0eservice_method\x18\x05 \x01(\t\x12\x14\n\x0cis_cancelled\x18\x06 \x01(\x08\x12\x11\n\tis_failed\x18\x07 \x01(\x08\"4\n\nPacketType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07REQUEST\x10\x01\x12\x0c\n\x08RESPONSE\x10\x02\"4\n\nPacketFlag\x12\x08\n\x04NULL\x10\x00\x12\x07\n\x03NEW\x10\x01\x12\n\n\x06\x43\x41NCEL\x10\x02\x12\x07\n\x03\x45ND\x10\x04\"&\n\x08Priority\x12\x07\n\x03LOW\x10\x00\x12\x07\n\x03MID\x10\x01\x12\x08\n\x04HIGH\x10\x02\"\x08\n\x06RpcAck\"\x06\n\x04Void\";\n\x0c\x45rrorMessage\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\tb\x06proto3')


_CONTEXT = DESCRIPTOR.message_types_by_name['Context']
_RPCACK = DESCRIPTOR.message_types_by_name['RpcAck']
_VOID = DESCRIPTOR.message_types_by_name['Void']
_ERRORMESSAGE = DESCRIPTOR.message_types_by_name['ErrorMessage']
_CONTEXT_PACKETTYPE = _CONTEXT.enum_types_by_name['PacketType']
_CONTEXT_PACKETFLAG = _CONTEXT.enum_types_by_name['PacketFlag']
_CONTEXT_PRIORITY = _CONTEXT.enum_types_by_name['Priority']
Context = _reflection.GeneratedProtocolMessageType('Context', (_message.Message,), {
    'DESCRIPTOR': _CONTEXT,
    '__module__': 'pymaid.rpc.pb.pymaid_pb2'
    # @@protoc_insertion_point(class_scope:pymaid.rpc.pb.Context)
})
_sym_db.RegisterMessage(Context)

RpcAck = _reflection.GeneratedProtocolMessageType('RpcAck', (_message.Message,), {
    'DESCRIPTOR': _RPCACK,
    '__module__': 'pymaid.rpc.pb.pymaid_pb2'
    # @@protoc_insertion_point(class_scope:pymaid.rpc.pb.RpcAck)
})
_sym_db.RegisterMessage(RpcAck)

Void = _reflection.GeneratedProtocolMessageType('Void', (_message.Message,), {
    'DESCRIPTOR': _VOID,
    '__module__': 'pymaid.rpc.pb.pymaid_pb2'
    # @@protoc_insertion_point(class_scope:pymaid.rpc.pb.Void)
})
_sym_db.RegisterMessage(Void)

ErrorMessage = _reflection.GeneratedProtocolMessageType('ErrorMessage', (_message.Message,), {
    'DESCRIPTOR': _ERRORMESSAGE,
    '__module__': 'pymaid.rpc.pb.pymaid_pb2'
    # @@protoc_insertion_point(class_scope:pymaid.rpc.pb.ErrorMessage)
})
_sym_db.RegisterMessage(ErrorMessage)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    _CONTEXT._serialized_start = 46
    _CONTEXT._serialized_end = 457
    _CONTEXT_PACKETTYPE._serialized_start = 311
    _CONTEXT_PACKETTYPE._serialized_end = 363
    _CONTEXT_PACKETFLAG._serialized_start = 365
    _CONTEXT_PACKETFLAG._serialized_end = 417
    _CONTEXT_PRIORITY._serialized_start = 419
    _CONTEXT_PRIORITY._serialized_end = 457
    _RPCACK._serialized_start = 459
    _RPCACK._serialized_end = 467
    _VOID._serialized_start = 469
    _VOID._serialized_end = 475
    _ERRORMESSAGE._serialized_start = 477
    _ERRORMESSAGE._serialized_end = 536
# @@protoc_insertion_point(module_scope)
